;
; Zentralen_Umschalter_x2_T10.asm
;
; Created: 29.06.2020 13:27:54
; Author : ballhause
;


;Tiny10 working registers 16 to 32 / Registers 0 to 15 do not exist
.def	Temp 			=r16  	
.def	TempH			=r17	
.def	Temp3			=r18	
.def	Null			=r19
.def	LED_Number		=r20


;Port setup
.equ	Charlieplex_Port	=PortB
.equ	Charlieplex_DIR		=DDRB
.equ	Charlie_0			=0
.equ	Charlie_1			=1
.equ	Charlie_2			=2


.dseg
.org 0x40



.cseg
.org 0x00
 rjmp	RESET		;Reset vector
 rjmp	RESET		;Int0 vector
 rjmp	RESET		;PinChange0 vector
 rjmp	RESET		;Timer0 Capture vector
 rjmp	RESET		;Timer0 Overflow vector
 rjmp	RESET		;Timer0 CompareA vector
 rjmp	RESET		;Timer0 CompareB vector
 rjmp	RESET		;Analog Comparator vector
 rjmp	WakeUp		;Watchdog Timeout vector
 rjmp	RESET		;VLM vector
 rjmp	RESET		;ADC vector



RESET:
 cli

 ldi	Temp, 0b00000000
 out	DDRB, Temp
 ldi	Temp, 0b00000000		
 out	PortB, Temp

 clr	Null

 ldi	Temp, 0xD8								;enable configuration change
 out	CCP, Temp
 ldi	Temp, 0b00000001
 out	CLKMSR, Temp							;internal 128kHz Oscillator as clock generator


;setup Watchdog
 ldi	Temp, (1<<WDP3)|(1<<WDIE)				;4s timeout
 out	WDTCSR, Temp

 ldi	Temp, 0b00000010						;disable ADC
 out	PRR, Temp
 
 ldi	Temp, (1<<ACD)							;disable Analog comparator
 out	ACSR, Temp
 
 ldi	Temp, 0x0F
 out	DIDR0, Temp

 clr	LED_Number

 sei 




MainLoop:
 ldi	Temp, (1<<SM1)|(1<<SE)
 out	SMCR, Temp

 sleep

 ldi	Temp, (1<<SM1)|(0<<SE)
 out	SMCR, Temp

 ldi	Temp3, 6
 

MainLoop1:
 rcall	Switch_LED  
 rcall	Delay_20ms
 rcall	LEDs_Off
 rcall	Delay_20ms
 inc	LED_Number
 dec	Temp3
 brne	MainLoop1

 dec	LED_Number
 ldi	Temp3, 6

MainLoop2:
 rcall	Switch_LED  
 rcall	Delay_20ms
 rcall	LEDs_Off
 rcall	Delay_20ms
 dec	LED_Number
 dec	Temp3
 brne	MainLoop2
 
 clr	LED_Number

 rjmp	MainLoop



;----------------------------------------------
Switch_LED:
 ldi	ZL, low(CharliePlexing_Table*2)
 ldi	ZH, high(CharliePlexing_Table*2)
 add	ZL, LED_Number
 adc	ZH, Null
 ldi	Temp, 0x40								;linear address space for Flash (0x4000-0x41FF)
 add	ZH, Temp
 ld		Temp, Z
 mov	TempH, Temp
 andi	TempH, 0x70
 swap	TempH
 out	Charlieplex_DIR, TempH
 andi	Temp, 0x07
 out	Charlieplex_PORT, Temp
 ret

LEDs_Off:
 out	Charlieplex_DIR, Null
 out	Charlieplex_PORT, Null
 ret


;----------------------------------------------
Delay_20ms:
 ldi	Temp, 40
 
_DelayLoop:
 dec	Temp
 brne	_DelayLoop
 ret


;-----------------------------------
;INTERRUPT SERVICE ROUTINES
;-----------------------------------


WakeUp:
 reti

;------------------------------------
CharliePlexing_Table:
.db	0b00110001, 0b00110010		;LED1 / LED2
.db 0b01100010, 0b01100100		;LED3 / LED4
.db	0b01010001, 0b01010100		;LED5 / LED6




//End of File//
